#!/usr/bin/env python
from __future__ import print_function

import glob, sys, os
dragonfly_sub_packages = (
  "dragonfly/*/__init__.py",
  "dragonfly/*/*/__init__.py",
  "dragonfly/*/*/*/__init__.py",
  "dragonfly/*/*/*/*/__init__.py",  
  "dragonfly/*/*/__init__.fly",
  "dragonfly/*/*/*/__init__.fly",
  "dragonfly/*/*/*/*/__init__.fly",  
)
dfdirs = []
for d in dragonfly_sub_packages:
  dfdirs += glob.glob(d)
dfdirs = [os.path.split(d)[0].replace('/','.') for d in dfdirs]
#remove duplicates
dfdirs0 = dfdirs
dfdirs = []
for d in dfdirs0:
  if d not in dfdirs: dfdirs.append(d)

assert len(dfdirs)

package_data = {'bee': 
  [
  'hivemap/*.*',
  'manual/*.*', 
  'manual/tetris/*.*',
  'manual/chess/*.*', 
  'manual/chess/models/*.*', 
  'manual/chess/components/*.*', 
  'manual/chess/components/drones/*.*', 
  'manual/chess/components/workers/*.*', 
  'manual/movingpanda/*.*',
  'tutorial/*.*',
  'tutorial/*/*.*',
  'tutorial/*/*/*.*',
  'tutorial/*/*/*/*.*',
  ],
  'gui': ['*.*'],    
}

for d in dfdirs:
  package_data[d] = ['*.spy', '*.fly', '*.txt', '*.workermap']

from distutils.core import setup
setup(name='hive',
  version='0.90a2',
  description='Component-based programming and editing framework',
  zip_safe = False,
  author='Sjoerd de Vries',
  author_email='sjdv1982@gmail.com',
  url='http://www.spyderware.nl/',
  packages=['bee', 'libcontext','dragonfly','hiveguilib', 
    'bee.segments', 'bee.spyderhive', 'bee.pin', 'bee.morph', 
    'dragonfly.spy',
    'hiveguilib.HGui', 'hiveguilib.HQt','hiveguilib.HQt.widgets', 
    'hiveguilib.PGui', 'hiveguilib.PQt', 
    'hiveguilib.worker', 'hiveguilib.segments',
    'hiveguilib.bees', 'hiveguilib.spyderbees',
    ] + dfdirs,
  package_data = package_data,
  scripts = [
   'gui/hivegui.py', 
   'gui/workergui.py', 
   'gui/spydergui.py', 
   'gui/hivegui-README.txt',
   'gui/locations.conf',
   'gui/hive-windows-postinstall',
   ],
  long_description = """A framework to build and connect extremely flexible, reusable Python components
The hive system can be used for visual programming, large 3D applications, game logic, and scientific protocols.""",
  classifiers =[
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python',
    'Development Status :: 3 - Alpha',
    'Operating System :: OS Independent',
  ],
)   
